import base64
import hashlib
import logging
import binascii
from Crypto.Cipher import AES

logger = logging.getLogger(__name__)


def md5(s: str, salt='', encoding='utf-8') -> str:
    return hashlib.md5((s + salt).encode(encoding=encoding)).hexdigest()


def base64Encode(s) -> bytes:
    s = str(s)
    return base64.b64encode(s.encode('utf-8'))


def base64Decode(s) -> str:
    s = str(s)
    return base64.b64decode(s).decode('utf-8')


def hex_to_bytes(s: str):
    return binascii.a2b_hex(s)


class CryptoModel:
    """
    AES加密类
    """

    def __init__(self, key, iv, mode=AES.MODE_CBC):
        self.key = md5(key).encode()
        self.iv = md5(iv).encode()[8:24]
        self.mode = mode

    @staticmethod
    def padding(text: bytes):
        padding_0a = (16 - len(text) % 16) * b' '
        return text + padding_0a

    def aes_encode(self, text: bytes):
        obj = AES.new(self.key, self.mode, self.iv)
        data = self.padding(text)
        return obj.encrypt(data)

    def aes_decode(self, data: bytes):
        obj = AES.new(self.key, self.mode, self.iv)
        return obj.decrypt(data)

    def encrypt(self, data):
        ' 加密函数 '
        cryptor = AES.new(self.key, self.mode, self.iv)
        return binascii.b2a_hex(cryptor.encrypt(data)).decode()

    def decrypt(self, data):
        ' 解密函数 '
        cryptor = AES.new(self.key, self.mode, self.iv)
        return cryptor.decrypt(binascii.a2b_hex(data)).decode()



from lxml import etree
import datetime


def xml_to_dict(root_or_str, strict=True):
    """
    Converts `root_or_str` which can be parsed xml or a xml string to dict.

    """
    root = root_or_str
    if isinstance(root, str):
        import xml.etree.cElementTree as ElementTree
        root = ElementTree.XML(root_or_str)
    return {root.tag: _from_xml(root, strict)}


def dict_to_xml(dict_xml):
    """
    Converts `dict_xml` which is a python dict to corresponding xml.
    """
    return _to_xml(dict_xml)


# Functions below this line are implementation details.
# Unless you are changing code, don't bother reading.
# The functions above constitute the user interface.

def _to_xml(el):
    """
    Converts `el` to its xml representation.
    """
    val = None
    if isinstance(el, dict):
        val = _dict_to_xml(el)
    elif isinstance(el, bool):
        val = str(el).lower()
    else:
        val = el
    if val is None: val = 'null'
    return val


def _extract_attrs(els):
    """
    Extracts attributes from dictionary `els`. Attributes are keys which start
    with '@'
    """
    if not isinstance(els, dict):
        return ''
    return ''.join(' %s="%s"' % (key[1:], value) for key, value in els.items()
                   if key.startswith('@'))


def _dict_to_xml(els):
    """
    Converts `els` which is a python dict to corresponding xml.
    """

    def process_content(tag, content):
        attrs = _extract_attrs(content)
        text = isinstance(content, dict) and content.get('#text', '') or ''
        return '<%s%s>%s%s</%s>' % (tag, attrs, _to_xml(content), text, tag)

    tags = []
    for tag, content in els.items():
        # Text and attributes
        if tag.startswith('@') or tag == '#text':
            continue
        elif isinstance(content, list):
            for el in content:
                tags.append(process_content(tag, el))
        elif isinstance(content, dict):
            tags.append(process_content(tag, content))
        else:
            tags.append('<%s>%s</%s>' % (tag, _to_xml(content), tag))
    return ''.join(tags)


def _str_to_datetime(date_str):
    try:
        val = datetime.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%SZ")
    except ValueError:
        val = date_str
    return val


def _str_to_boolean(bool_str):
    if bool_str.lower() != 'false' and bool(bool_str):
        return True
    return False


def _from_xml(el, strict):
    """
    Extracts value of xml element element `el`.
    """
    val = None
    # Parent node.
    if len(el):
        val = {}
        for e in el:
            tag = e.tag
            v = _from_xml(e, strict)
            if tag in val:
                # Multiple elements share this tag, make them a list
                if not isinstance(val[tag], list):
                    val[tag] = [val[tag]]
                val[tag].append(v)
            else:
                # First element with this tag
                val[tag] = v
    # Simple node.
    else:
        attribs = el.items()
        # An element with attributes.
        if attribs and strict:
            val = dict(('@%s' % k, v) for k, v in dict(attribs).items())
            if el.text:
                converted = _val_and_maybe_convert(el)
                val['#text'] = el.text
                if converted != el.text:
                    val['#value'] = converted
        elif el.text:
            # An element with no subelements but text.
            val = _val_and_maybe_convert(el)
        elif attribs:
            val = dict(attribs)
    return val


def _val_and_maybe_convert(el):
    """
    Converts `el.text` if `el` has attribute `type` with valid value.
    """
    text = el.text.strip()
    data_type = el.get('type')
    convertor = _val_and_maybe_convert.convertors.get(data_type)
    if convertor:
        return convertor(text)
    else:
        return text


_val_and_maybe_convert.convertors = {
    'boolean': _str_to_boolean,
    'datetime': _str_to_datetime,
    'integer': int
}


def read_file(file):
    with open(file) as f:
        data = f.read()
        data = bytes(bytearray(data, encoding='utf-8'))
    root = etree.XML(data)
    return xml_to_dict(root)


def get_res(file):
    tmp = read_file(file)
    res = {}
    for i, item in enumerate(tmp['result']['security']):
        res[i] = []
        for cmd in item['item']:
            res[i].append(cmd['cmd'])
    return res


import queue
from queue import Full
from time import monotonic as time


class UniqueQueue(queue.Queue):
    def put(self, item, block=True, timeout=None, unique=False):
        """增加了unique参数"""
        with self.not_full:
            # ----- 以下三行为新增代码 -----#
            if unique:
                if item in self.queue:
                    return
            # ----- 新增代码结束 -----#
            if self.maxsize > 0:
                if not block:
                    if self._qsize() >= self.maxsize:
                        raise Full
                elif timeout is None:
                    while self._qsize() >= self.maxsize:
                        self.not_full.wait()
                elif timeout < 0:
                    raise ValueError("'timeout' must be a non-negative number")
                else:
                    endtime = time() + timeout
                    while self._qsize() >= self.maxsize:
                        remaining = endtime - time()
                        if remaining <= 0.0:
                            raise Full
                        self.not_full.wait(remaining)
            self._put(item)
            self.unfinished_tasks += 1
            self.not_empty.notify()
